function [F,x,it,time,flag] = InexactNewton(A,b,n,sp,x,theta,tol)

% This is Inexact semi-smooth Newton method for Absolute Value Equations 
% (AVE): Ax - |x| = b

% Reference: J.Y. Bello Cruz, O. P. Ferreira, L. F. Prudente. On the global
%            convergence of the inexact semi-smooth Newton method for 
%            absolute value equation

%--------------------- Input arguments ------------------------------------

% n: dimension of the problem

% A, b: data of the problem Ax - |x| = b; 
%       A is nxn real matriz and b is nx1 real vector 

% x: initial point; nx1 real vector 

% sp: sparsity indicator
%     if sp  = 0: full storage organization is used
%     if sp ~= 0: sparse storage organization is used

% theta: tolerance factor for Inexact Newton subproblems

% tol: accuracy for solution of AVE
%      The method stops at x if ||Ax - |x| - b|| <= tol


%--------------------- Output arguments -----------------------------------

% x: final iterate

% F: norm of AVE at the final iterate x; 
%    F = ||Ax - |x| - b||

% it: number of Inexact Newton iterations

% time: Total CPU time in seconds

% flag = -1: an error occurred during the execution
% flag =  0: solution was found
% flag =  1: maximum of iterations reached

%-----------------------  Dependence --------------------------------------

% function: SolveLinearSystem


%-------------- Iterative method for linear equations ---------------------

% LSopt indicates the iterative method for linear equations 

% LSopt = 1: bicg        - BiConjugate Gradients Method
% LSopt = 2: bicgstab    - BiConjugate Gradients Stabilized Method
% LSopt = 3: bicgstabl   - BiCGStab(l) Method
% LSopt = 4: cgs         - Conjugate Gradients Squared Method
% LSopt = 5: gmres       - Generalized Minimum Residual Method
% LSopt = 6: lsqr        - LSQR Method
% LSopt = 7: qmr         - Quasi-Minimal Residual Method
% LSopt = 8: tfqmr       - Transpose-Free QMR Method

LSopt = 6;


% Print initial information

disp('-------------------------------------------------------------------')
disp('------------------   Initial Information   ------------------------')
disp('-------------------------------------------------------------------')
disp(' ')
disp(' ')
disp('This is Inexact semi-smooth Newton method for Absolute Value Equations')
disp('(AVE): Ax - |x| = b   ')
disp(' ')
disp('Reference: J.Y. Bello Cruz, O. P. Ferreira, L. F. Prudente. On the') 
disp('global convergence of the inexact semi-smooth Newton method for') 
disp('absolute value equation')
disp(' ')
fprintf('Dimension of matrix A (nxn) = %d \n',n)
fprintf('Tolerance for solution of AVE = %6.2e \n',tol)
fprintf('Tolerance factor for Inexact Newton subproblems, theta = %5.4f % \n',theta)
disp(' ')
disp(' ')
disp('-------------------------------------------------------------------')
disp('-------------   Inexact semi-smooth Newton method -----------------')
disp('-------------------------------------------------------------------')
disp(' ')

if LSopt == 1
    disp('Iterative LS Method: bicg        - BiConjugate Gradients Method')
elseif LSopt == 2
    disp('Iterative LS Method: bicgstab    - BiConjugate Gradients Stabilized Method')
elseif LSopt == 3
    disp('Iterative LS Method: bicgstabl   - BiCGStab(l) Method')
elseif LSopt == 4
    disp('Iterative LS Method: cgs         - Conjugate Gradients Squared Method')
elseif LSopt == 5
    disp('Iterative LS Method: gmres       - Generalized Minimum Residual Method')
elseif LSopt == 6
    disp('Iterative LS Method: lsqr        - LSQR Method')
elseif LSopt == 7
    disp('Iterative LS Method: qmr         - Quasi-Minimal Residual Method')
elseif LSopt == 8
    disp('Iterative LS Method: tfqmr       - Transpose-Free QMR Method')
end

disp(' ')
disp(' It   ||Ax - |x| - b||       LStol    LSSit   flagLSS')
disp(' ')

tic;
        
flag = -1;

F = norm( A*x - abs(x) -b );
normb = norm(b);

maxit = 50;
it = 0;

fprintf(' %3d       %6.2e\n',it,F)

while F > tol
    
    if it >= maxit
        flag = 1;
        
        time = toc;
        
        disp('Inexact Newton method fails to find the solution')
        disp('Maximum of iterations reached ')
        
        return
    end
    
    it = it + 1;

    LStol = theta * F / normb;

    if ( sp == 0 )
        AD = A - diag(sign(x));
    else    
        AD = A-sparse(1:n,1:n,sign(x),n,n);
    end

    [x,flagLS,relres,iter] = SolveLinearSystem(AD,b,n,x,LStol,LSopt);
    
    F = norm( A*x - abs(x) -b );
    
    % Print information
    
    fprintf(' %3d       %6.2e         %6.2e    %4d      %d \n',it,F,LStol,iter,flagLS)

end

flag = 0;
time = toc;

disp(' ')
disp('Solution was found')
fprintf('Time = %10.4f \n',time)